/***********************************************************************
 *                                                                     *
 *          SAC ( SEISMIC ANAKYSIS CODE  ) DATA FILE ACCESS INTERFACE  *
 *                                                                     *
 *          DEVISED  BY    F.L NIU         1993-03-23                  * 
 *          CODED    BY    F.L NIU         1993-03-23                  * 
 *          REVISED  BY    F.L NIU         1993-04-30                  * 
 *                                                                     *
 ***********************************************************************
*/


/* constant for sac header and sac file              */  

#define               VL_MAX 80001
#define               HDMAX    134
#define               INTE4      2
#define               FLOAT      1
#define               CHAR8      3
#define               CHAR16     4
#define               DEFAULT_FLOAT         -12345.
#define               DEFAULT_INTE4         -12345
#define               DEFAULT_CHAR          "-12345  "
#define               ON         1
#define               OFF        0
#define               ALL        1
#define               PART       2
#define               HD_FLOAT_LEN   70
#define               HD_INTE4_LEN   40
#define               HD_CHAR_LEN    24

/* constant for IASP91 travel time                  */

static  struct    SACHD   {
   float   r_hd[HD_FLOAT_LEN];
   int     n_hd[HD_INTE4_LEN];
   char    a_hd[HD_CHAR_LEN][8];
};

/* constant for plot record section program (plin91) */

static  struct   SACINDX    {
   char      hdnm[16];
   int       type;
   int       hdn;
}sacindx[HDMAX]    = {      
                      {  "delta",  FLOAT,  1},   /* sampling interval        */  
                      {  "depmin", FLOAT,  2},   /* Minimum  value           */  
                      {  "depmax", FLOAT,  3},   /* Maximum value            */  
                      {  "scale",  FLOAT,  4},   /* Scale factor             */  
                      {  "odelta", FLOAT,  5},   /* Observed interval        */  
                      {  "b",      FLOAT,  6},   /* Begining value           */  
                      {  "e",      FLOAT,  7},   /* Ending value             */  
                      {  "o",      FLOAT,  8},   /* Event original time      */  
                      {  "a",      FLOAT,  9},   /* First arrival time       */  
                      {  "t0",     FLOAT, 11},   /* T0                       */
                      {  "t1",     FLOAT, 12},   /* T1                       */
                      {  "t2",     FLOAT, 13},   /* T2                       */
                      {  "t3",     FLOAT, 14},   /* T3                       */
                      {  "t4",     FLOAT, 15},   /* T4                       */
                      {  "t5",     FLOAT, 16},   /* T5                       */
                      {  "t6",     FLOAT, 17},   /* T6                       */
                      {  "t7",     FLOAT, 18},   /* T7                       */
                      {  "t8",     FLOAT, 19},   /* T8                       */
                      {  "t9",     FLOAT, 20},   /* T9                       */
                      {  "f",      FLOAT, 21},   /* Fini or end of event time*/
                      {  "stla",   FLOAT, 32},   /* Station latitude         */  
                      {  "stlo",   FLOAT, 33},   /* Station longtitude       */  
                      {  "stel",   FLOAT, 34},   /* Station elevation  (m)   */
                      {  "stdp",   FLOAT, 35},   /* Station depth (m)        */  
                      {  "evla",   FLOAT, 36},   /* Event   latitude         */  
                      {  "evlo",   FLOAT, 37},   /* Event   longtitude       */  
                      {  "evel",   FLOAT, 38},   /* Event   elevation  (m)   */
                      {  "evdp",   FLOAT, 39},   /* Event   depth (m)        */  
                      {  "dist",   FLOAT, 51},   /* Station to event distance*/
                      {  "az",     FLOAT, 52},   /* Event to Station azimuth */
                      {  "baz",    FLOAT, 53},   /* Station to event azimuth */
                      {  "gcarc",  FLOAT, 54},   /* Station to event arc dist*/
                      {  "depmen", FLOAT, 57},   /* Mean    value            */  
                      {  "cmpaz",  FLOAT, 58},   /* Component azimuth        */  
                      {  "cmpinc", FLOAT, 59},   /* Component incident angle */
                      {  "nzyear", INTE4 , 1},   /* Year                     */  
                      {  "nzjday", INTE4,  2},   /* Julian day               */  
                      {  "nzhour", INTE4,  3},   /* Hour                     */  
                      {  "nzmin",  INTE4,  4},   /* Minute                   */  
                      {  "nzsec",  INTE4,  5},   /* Second                   */  
                      {  "nzmsec", INTE4,  6},   /* milisecond               */  
                      {  "nvhdr",  INTE4,  7},   /* Header version           */  
                      {  "npts",   INTE4, 10},   /* Total sampling number    */
                      {  "iftype", INTE4, 16},   /* Type of  file            */
                      {  "idep",   INTE4, 17},   /* Type of vaiable          */  
                      {  "iztype", INTE4, 18},   /* reference time flag      */  
                      {  "iinst",  INTE4, 20},   /* type of recording system */
                      {  "ievtyp", INTE4, 23},   /* Type of event            */  
                      {  "iqual",  INTE4, 24},   /* Quality of data          */  
                      {  "isynth", INTE4, 25},   /* Synthetic data flag      */  
                      {  "leven",  INTE4, 36},   /* evenly spaced flag       */  
                      {  "lpspol", INTE4, 37},   /* positive polarity flag   */
                      {  "lovrok", INTE4, 38},   /* write flag               */  
                      {  "lcalda", INTE4, 39},   /*                          */
                      {  "ko",     CHAR8,  5},   /* Event origin time id     */  
                      {  "ka",     CHAR8,  6},   /* First arrival time ID    */
                      {  "kf",     CHAR8, 17},   /* Fini Id                  */  
                      {  "kinst",  CHAR8, 24},   /* recording instrument     */  
                      {  "knetwk", CHAR8, 22},   /* name of seismic network  */
                      {  "kstnm",  CHAR8,  1},   /* station name             */  
                      {  "kcmpnm", CHAR8, 21},   /* Component name           */  
                      {  "kevnm",  CHAR16, 2},   /* event name               */  
                      {  "khole",  CHAR8,  4},   /* Hole Id of nucler event  */
                      {  "kdatrd", CHAR8, 23}    /* Date data was read       */  
                 };
struct    SACHDNM  {
   float            delta;
   float            depmin;
   float            depmax;
   float            scale;
   float            odelta;
   float            b;
   float            e;
   float            o;
   float            a;
   float            internal1;
   float            t0;
   float            t1;
   float            t2;
   float            t3;
   float            t4;
   float            t5;
   float            t6;
   float            t7;
   float            t8;
   float            t9;
   float            f;
   float            resp0;
   float            resp1;
   float            resp2;
   float            resp3;
   float            resp4;
   float            resp5;
   float            resp6;
   float            resp7;
   float            resp8;
   float            resp9;
   float            stla;
   float            stlo;
   float            stel;
   float            stdp;
   float            evla;
   float            evlo;
   float            evel;
   float            evdp;
   float            unused1;
   float            user[9];
   float            dist;
   float            az;
   float            baz;
   float            gcarc;
   float            internal2[2];
   float            depmen;
   float            cmpaz;
   float            cmpinc;
   float            unused2[11];
   int              nzyear;
   int              nzjday;
   int              nzhour;
   int              nzmin;
   int              nzsec;
   int              nzmsec;
   int              internal3[3];
   int              npts;
   int              internal4[2];
   int              unused3[3];
   int              iftype;
   int              idep;
   int              iztype;
   int              unused4;
   int              iinst;
   int              istreg;
   int              ievreg;
   int              ievtyp;
   int              iqual;
   int              isynth;
   int              unused5[10];
   int              leven;
   int              lpspol;
   int              lovrok;
   int              lcalda;
   int              unused6;
   char             kstnm[8];
   char             kevnm[16];
   char             khole[8];
   char             ko[8];
   char             ka[8];
   char             kt0[8];
   char             kt1[8];
   char             kt2[8];
   char             kt3[8];
   char             kt4[8];
   char             kt5[8];
   char             kt6[8];
   char             kt7[8];
   char             kt8[8];
   char             kt9[8];
   char             kf[8];
   char             kuser[3][8];
   char             kcmpnm[8];
   char             knetwk[8];
   char             kdatrd[8];
   char             kinst[8];
};
