#include <stdio.h>
#include <math.h>
deltaz_(slat,slon,elat,elon,delt,dist,azim,bazim)
float slat, slon, elat, elon;
float *delt, *dist, *azim, *bazim;

/* ***************************************************************************
  This subroutine calculates four quantities that depend on the  
  relative location on the globe of epicenter and station.          
INPUT: slat(station latitude), slon(station longitude), elat(epi-  
  center latitude), and elon(epicenter longitude).  Latitudes(which  
  are input as geographic latitudes) are to be given from 0 to 90    
  degrees (+ for North, and - for South).  Longitudes    
  are to be given from -180 to +180 degrees (+ for East and - for West),
  or from 0 to 360 degrees (to the East) with 0=Greenwich Meridian.
OUTPUT: delt(angular separation in degrees), dist(geodesic distance
  in kilometers), azim(epicenter-to-station azimuth, in degrees N    
  thru E), and bazim(station-to-epicenter azimuth, in degrees N thru E).
*************************************************************************** */


{
double slatr,slonr,elatr,elonr,sclatr,eclatr,
       aaa,a,b,c,a1,b1,c1,seclat,ceclat,selon,celon,
       ssclat,csclat,sslon,cslon,cd,cz,z,cbz,bz;

/* converting input angles from degrees to radians
   (suffix r always denotes radian measure)  */
slatr = .01745329*slat;
slonr = .01745329*slon;
elatr = .01745329*elat;
elonr = .01745329*elon;


/* converting geographic latitudes to geocentric latitudes 
aaa = 0.996647*tan(slatr);
slatr = atan(aaa);
aaa = 0.996647*tan(elatr);
elatr = atan(aaa);
*/
aaa = 0.99330524*tan(slatr);
slatr = atan(aaa);
aaa = 0.99330524*tan(elatr);
elatr = atan(aaa);


/* At this point latitudes are converted into colatitudes, which run  
  from 0 at North Pole to 180 at South Pole.  Also, longitudes are   
  re-evaluated so that they run from 0 to 360 from Greenwich Eastward.
  sclatr=station colatitude   eclatr=epicenter colatitude  (radians) */
sclatr = 1.570796327-slatr;
if (slonr < 0.)   slonr += 6.283185307;
eclatr = 1.570796327-elatr;
if (elonr < 0.)   elonr += 6.283185307;
seclat = sin(eclatr);
ceclat = cos(eclatr);
selon = sin(elonr);
celon = cos(elonr);
ssclat = sin(sclatr);
csclat = cos(sclatr);
sslon = sin(slonr);
cslon = cos(slonr);

/* calculation of direction cosines of station (a1,b1,c1) and of    
  epicenter (a,b,c). For this calculation, x-axis is thru Greenwich  
  Meridian, y-axis at 90E longitude, and z-axis thru North Pole.   */
a = seclat*celon;
b = seclat*selon;
c = ceclat;
a1 = ssclat*cslon;
b1 = ssclat*sslon;
c1 = csclat;

/* Now you have all the info necessary to compute delt, dist, azim,
  and bazim.  For reference:  cosine of delta=cd, cosine of azimuth=cz
  azim. in radians=z, cosine of backazimuth=cbz, and back-azimuth
  in radians=bz.                    */
cd = a*a1 + b*b1 + c*c1;
cd = (cd > 1.) ? 1. : cd;
cd = (cd < -1.) ? -1. : cd;
*delt = acos(cd)*57.29577951;
*dist = *delt*1.745329252e-2*6371.0;

/* computation of cz and z.  The following standard
   formula is derivable via spherical geometry.  */
cz = (seclat*csclat-ceclat*ssclat*(celon*cslon+selon*sslon))/sqrt(1.-cd*cd);
cz = (cz > 1.) ? 1. : cz;
cz = (cz < -1.) ? -1. : cz;
z = acos(cz);

/* The following test determines whether z should be > or < 180 degrees.*/
if ((sslon*celon-cslon*selon) < 0.) z=6.283185307-z;
*azim = z*57.29577951;
	
/* computation of cbz and bz.  This is accomplished by switching the
  roles of station and epicenter in the previous formula. */
cbz = (ssclat*ceclat-csclat*seclat*(cslon*celon+sslon*selon))/sqrt(1.-cd*cd);
cbz = (cbz > 1.) ? 1. : cbz;
cbz = (cbz < -1.) ? -1. : cbz;
bz = acos(cbz);

/* The following test determines whether bz should be > or < 180 degrees.*/
if ((selon*cslon-celon*sslon) < 0.) bz=6.283185307-bz;
*bazim = bz*57.29577951;
}

midp_(elat, elon, slat, slon, mlat, mlon, plat, plon, np)
float   elat, elon, slat, slon;
float   *mlat, *mlon;
float   *plat, *plon;
int     *np;
{
   double slatr,slonr,elatr,elonr,sclatr,eclatr,
       aaa,a,b,c,a1,b1,c1,seclat,ceclat,selon,celon,
       ssclat,csclat,sslon,cslon, am, bm, cm, rm2, mlatr,
       mclatr, mlonr, aa,bb,cc,ase,bse,cse,latr,clatr,lonr;
   float delt, dist, azim, bazim;
   int   n, i;
   
   slatr = .01745329*slat;
   slonr = .01745329*slon;
   elatr = .01745329*elat;
   elonr = .01745329*elon;
   /* converting geographic latitudes to geocentric latitudes */
   aaa = 0.996647*tan(slatr);
   slatr = atan(aaa);
   aaa = 0.996647*tan(elatr);
   elatr = atan(aaa);
   
/* At this point latitudes are converted into colatitudes, which run
  from 0 at North Pole to 180 at South Pole.  Also, longitudes are
  re-evaluated so that they run from 0 to 360 from Greenwich Eastward.
  sclatr=station colatitude   eclatr=epicenter colatitude  (radians) */
  sclatr = 1.570796327-slatr;
  if (slonr < 0.)   slonr += 6.283185307;
  eclatr = 1.570796327-elatr;
  if (elonr < 0.)   elonr += 6.283185307;
   seclat = sin(eclatr);
   ceclat = cos(eclatr);
   selon = sin(elonr);
   celon = cos(elonr);
   ssclat = sin(sclatr);
   csclat = cos(sclatr);
   sslon = sin(slonr);
   cslon = cos(slonr);
/* calculation of direction cosines of station (a1,b1,c1) and of
  epicenter (a,b,c).  this calculation, x-axis is thru Greenwich
  Meridian, y-axis at 90E longitude, and z-axis thru North Pole. 
  tThe mid-point ls int the direction of (a+a1, b+b1, c+c1)  */
  a = seclat*celon;
  b = seclat*selon;
  c = ceclat;
  a1 = ssclat*cslon;
  b1 = ssclat*sslon;
  c1 = csclat;
  am = a + a1;
  bm = b + b1;
  cm = c + c1;
  
  rm2 = am * am + bm * bm + cm *cm;
  rm2 = sqrt(rm2);
  am = am / rm2;
  bm = bm / rm2;
  cm = cm / rm2;
  
cm = (cm > 1.) ? 1. : cm;
cm = (cm < -1.) ? -1. : cm;
  mclatr = acos(cm);
  mlatr = 1.570796327 - mclatr;
/* converting geocentric latitudes to  geographic latitudes  */
  aaa = tan(mlatr) / 0.996647;
  mlatr = atan(aaa);
  *mlat  = 57.29577951 * mlatr;
  mlonr = atan2(bm, am);
  if( mlonr < 0.)
    mlonr += 6.283185307;
  *mlon = mlonr * 57.29577951; 
  deltaz_(slat,slon,elat,elon,&delt,&dist,&azim,&bazim);
  n = (int)delt;
  *np = n + 1;
  ase  = (a1 - a)/ n;
  bse  = (b1 - b)/ n;
  cse  = (c1 - c)/ n;
  for(i = 0; i < n; i++) 
  {
     aa = a + i * ase;
     bb = b + i * bse;
     cc = c + i * cse;
     rm2 = aa * aa + bb * bb + cc * cc;   
     rm2 =sqrt(rm2);
     aa = aa / rm2;
     bb = bb / rm2;
     cc = cc / rm2;
cc = (cc > 1.) ? 1. : cc;
cc = (cc < -1.) ? -1. : cc;
     clatr = acos(cc);
     latr = 1.570796327 - clatr;
     aaa = tan(latr) / 0.996647;
     latr = atan(aaa);
     plat[i]  = 57.29577951 * latr;
     lonr = atan2(bb, aa);
     if( lonr < 0.)
        lonr += 6.283185307;
     plon[i] = lonr * 57.29577951;
  }
  plat[n] = slat;
  plon[n] = slon;
}
gcarc_pos_(Glat, Glon, Rlat, Rlon, delt, Xlat, Xlon)
        float Glat, Glon, Rlat, Rlon, delt;
        float *Xlat, *Xlon;
{
	double	Rlatr, Rlonr, Glatr, Glonr, deltr, Rclatr, Gclatr;
	double	sGclat, cGclat, sGlon, cGlon;
	double	sRclat, cRclat, sRlon, cRlon;
	double	sdelt, cdelt;
	double  a0, b0, c0, a1, b1, c1, a2, b2, c2;
	double	ax, bx, cx;

        *Xlat = 0.;
        *Xlon = 0.;

        /* converting input angles from degrees to radians
         (suffix r always denotes radian measure)  */
        Rlatr = .01745329*Rlat;
        Rlonr = .01745329*Rlon;
        Glatr = .01745329*Glat;
        Glonr = .01745329*Glon;
	deltr = .01745329*delt;

/* At this point latitudes are converted into colatitudes, which run
  from 0 at North Pole to 180 at South Pole.  Also, longitudes are
  re-evaluated so that they run from 0 to 360 from Greenwich Eastward.
  sclatr=station colatitude   eclatr=epicenter colatitude  (radians) */
	Rclatr = 1.570796327-Rlatr;
	if (Rlonr < 0.)   Rlonr += 6.283185307;
	Gclatr = 1.570796327-Glatr;
	if (Glonr < 0.)   Glonr += 6.283185307;
	sGclat = sin(Gclatr);
	cGclat = cos(Gclatr);
	sGlon = sin(Glonr);
	cGlon = cos(Glonr);
	sRclat = sin(Rclatr);
	cRclat = cos(Rclatr);
	sRlon = sin(Rlonr);
	cRlon = cos(Rlonr);
	sdelt = sin(deltr);
	cdelt = cos(deltr);

/* calculation of direction cosines of Reference Point (a1,b1,c1) and of
  G. C. Pole (a0,b0,c0). For this calculation, x-axis is thru Greenwich
  Meridian, y-axis at 90E longitude, and z-axis thru North Pole.   */
	a0 = sGclat*cGlon;
	b0 = sGclat*sGlon;
	c0 = cGclat;
	a1 = sRclat*cRlon;
	b1 = sRclat*sRlon;
	c1 = cRclat;
/* Now calculate a2 which is the vector mutiple of a x a1 */
	a2 =	b0*c1 - c0*b1;
	b2 =	c0*a1 - a0*c1;
	c2 =	a0*b1 - b0*a1;

/*
printf("%10.6lf\n", a2*a2 + b2*b2 + c2*c2);
*/
	ax = a1 * cdelt + a2 * sdelt;
	bx = b1 * cdelt + b2 * sdelt;
	cx = c1 * cdelt + c2 * sdelt;
	
	*Xlat = 90. - acos(cx) * 57.29577951;
	*Xlon = atan2(bx, ax) * 57.29577951;
	if(*Xlon < 0.) *Xlon += 360.;
}

