/**********************************************************************
 *                                                                    *
 *    These subroutines are used when you want read a sac file or     *
 * get the header information.                                        *
 *                                                                    *
 *                          F.L NIU  1993-01-21                       *
 *                                                                    *
 **********************************************************************
*/

#include  <stdio.h>
#include  <math.h>
#include  <ctype.h>
#include   "sac.h" 

/**********************************************************************
 * get_hd_    = get a header content named hd_name to variable        *
 *              hd_buf. The name of the header contents is de-        *
 *              fined in sac.h.                                       *
 *                                                                    *
 *       calling sequence:                                            *
 *                                                                    *
 *           INPUT :                                                  *
 *                   1    sac_fil         sac_file  name              *
 *                   2    hd_name         name of a header content    *
 *           OUTPUT:                                                  *
 *                   1    hd_buf          head variable buffer        *
 *                   2    nerr            return code                 *
 *                                                                    *
 *                           DEVISED BY    F.L. NIU 1993-01-21        *
 *                           CODED   BY    F.L. NIU 1993-01-21        *
 *         (VERSION 1)       REVIDED BY    F.L. NIU 1993-06-25        *
 *                                                                    *
 *   In Version 1. The header pararmetses which is not inlcuded       *
 * is caulculated out by the basic ones. when there is a error        *
 * value in header, this is very usefull                              *
 *                                                                    *
 **********************************************************************
*/
               
static  int   hdntbl[8] = { 2, 3, 57, 7, 51, 52, 53, 54};
int get_hd_(sac_fil, hd_name, hd_buf, nerr)
char     *sac_fil;              /* sac file name            */
char     *hd_name;              /* head variable name       */
void     *hd_buf;               /* head variable buffer     */
int      *nerr;
{
   FILE       *fpin;
   struct    SACHD    shd;
   int      nc, mc, hdn,  nlen;
   int      find;

   find = OFF;
   *nerr = 0;
   if((fpin = fopen(sac_fil, "r")) == NULL)
   {
      fprintf(stderr,  "%s  FILE OPEN ERROR\n", sac_fil);
      *nerr = -1;
      return(-1);
   }
   fread(&shd,sizeof(shd),1,fpin);
   fclose(fpin);
   nlen = strlen(hd_name);
   for(nc = 0; nc < nlen; nc++ )
      hd_name[nc] = tolower(hd_name[nc]);
   for(nc = 0; nc < HDMAX; nc++)
   {
      if(!charcmp(hd_name, sacindx[nc].hdnm))
      {
         find  = ON;
         hdn = sacindx[nc].hdn - 1;
         switch(sacindx[nc].type)   {
            case     FLOAT :
                            memcpy(hd_buf, &shd.r_hd[hdn], 4);
                            for(mc = 0; mc < 8; mc++)
                            {
                               if(sacindx[nc].hdn == hdntbl[mc])
                               {
                                  cal_hd_(sac_fil, mc+1, hd_buf);
                                  break;
                               }
                            }
                            break;
            case     INTE4 :
                            memcpy(hd_buf, &shd.n_hd[hdn], 4);
                            break;
            case     CHAR8 :
                            memcpy(hd_buf, shd.a_hd[hdn], 8);
									 shd.a_hd[hdn][7] = '\0';
                            break;
            case     CHAR16 :
                            memcpy(hd_buf, shd.a_hd[hdn], 16);
									 shd.a_hd[hdn][15] = '\0';
                            break;
            default         :
                            break;
         }
         break;
      }
   }
   if(find == OFF)
   {
      fprintf(stderr, "%s IS NOT A CORRECT HEAD VARIABLE NAME  !!\n",
              hd_name);
      *nerr = 1;
		return(1);
   }
	return(0);
}
/*
 *     cal_hd( ) = 
*/
cal_hd_(sac_fil, nd, hd_buf)
char      *sac_fil;
int       nd;
float     *hd_buf;
{
   int       nerr;
   int       npts;
   int       nv; 
   float    rb0, bt, et, delta;
   float    evla, evlo;
   float    stla, stlo;
   float    vl[VL_MAX];
   float    f_buf[8]; 
   
   get_hd_(sac_fil, "b", &rb0, &nerr);
   get_hd_(sac_fil,  "npts", &npts, &nerr);
   get_hd_(sac_fil, "delta", &delta, &nerr);
   get_hd_(sac_fil, "evla", &evla, &nerr);
   get_hd_(sac_fil, "evlo", &evlo, &nerr);
   get_hd_(sac_fil, "stla", &stla, &nerr);
   get_hd_(sac_fil, "stlo", &stlo, &nerr);
   deltaz_(stla, stlo, evla, evlo,
               &f_buf[7], &f_buf[4], &f_buf[5], &f_buf[6]);
   f_buf[3] = rb0 + ( npts - 1) * delta; 
if(nd == 2 || nd == 3 || nd == 57) {
   rd_sac_(sac_fil, ALL, &bt, &et, vl, &nv);
   maxmin_(vl, nv, &f_buf[0], &f_buf[1], &f_buf[2]);
}
   *hd_buf = f_buf[nd -1];
}   
      
/**********************************************************************
 * rd_sac_    = read an evenly sampled sac file according to the      *
 *              read_flag                                             *
 *                                                                    *
 *       calling sequence:                                            *
 *                                                                    *
 *           INPUT :                                                  *
 *                   1    sac_fil         sac_file  name              *  
 *                   2    rd_flg  =       1 : read  all the data      *
 *                                        2 : read part of the data   * 
 *                   3    bt                  from bt to et           *
 *                   4    et                                          * 
 *           OUTPUT:                                                  *
 *                   1    vl              sac data                    *
 *                   2    nt              total read points           *
 *                   3    bt              return as Begin value when  *
 *                                        rd_flag = all               *
 *                   4    et              return as end   value when  *
 *                                        rd_flag = all               *
 *                                                                    *
 **********************************************************************
*/
rd_sac_(sac_fil,  rd_flg,  bt, et, vl, nt)
char    *sac_fil;
int     rd_flg;            /* read flag  all --> 1   part  --> 2 */
float   *bt;               /* I/O   O : for all ; I : for part   */
float   *et;               /* I/O   O : for all ; I : for part   */
float   *vl;
int     *nt;
{
   FILE      *fpsac;
   struct    SACHD    shd;
   int       npts, nlen, hdlen, nc, nspt, nerr;
   float     delta, rb0, tlen;

   get_hd_(sac_fil, "npts", &npts, &nerr);
   get_hd_(sac_fil, "delta", &delta, &nerr);
   get_hd_(sac_fil, "b",    &rb0, &nerr);
   if( rd_flg == ALL)
   {
      *bt = rb0;
      *et = rb0 + (npts - 1) * delta;
   }else{
      if(*bt <rb0) {
         *bt = rb0;
      }
      if(*et >=rb0 + (npts - 1) * delta) {
         *et = rb0 + (npts - 1) * delta;
      }
   } 
   tlen = *et - *bt;
/*
   nlen = ceil( (tlen / delta ) + 1;
*/
   nlen = (int)ceil(tlen/delta+0.0003);
/*
printf("rd_sac: %s %10.4e %10.4e %10.4e %10.4e %10.5e %d\n", sac_fil,
		*et, *bt, tlen, tlen / delta, delta, nlen);
*/
   npts = (nlen < npts) ? nlen : npts;
   for(nc = 0; nc < npts; nc++)
      vl[nc] = 0.;
   if((fpsac = fopen(sac_fil, "r")) == NULL)
   {
      fprintf(stderr,  "%s  FILE OPEN ERROR\n", sac_fil);
      exit();
   }
   hdlen = sizeof(shd);
   nspt = (*bt - rb0) / delta + 0.5;
#ifdef SAC_DEBUG
printf("rd_sac: %s %10.4f %10.4f %10.4f %d\n", sac_fil
	*bt, rb0, (*bt - rb0) / delta + 0.5, nspt);
#endif
/*
   if(*bt < rb0)
   {
      nspt = 0;
      tlen = *et - rb0;
      nlen = ( tlen + 1.e-4 ) / delta + 1;
      npts = (nlen < npts) ? nlen : npts;
      npts = (npts < 0) ? 0 : npts;
      *bt  = rb0;
      *et = (npts - 1) * delta + rb0;
   } 
*/
   *nt = npts; 
   fseek(fpsac, nspt*4 + hdlen, 0);
   fread(vl, 4, npts, fpsac) ;    /* read sac data   */
   fclose(fpsac);
}

rd_sac0_(sac_fil, bt, tl, nomal,  nt, vl)
char   *sac_fil;
float   bt, tl;
int     nomal;
int     *nt;
float   *vl;
{
   FILE      *fpsac;
   struct    SACHD    shd;
   int       npts, nv, nc, nerr;
   float     bt0, et0, st, et;
   float     vlmax, vlmin, vlmen, rw;
   float     depmin, depmax, depmen;
   float     buf[VL_MAX];
   float     yy;
   
   st = bt;
   et = bt + tl;
   rd_sac_(sac_fil, PART, &st, &et, vl, nt);
   npts = *nt;
   switch(nomal)  {
      case    1:   /* normalized by the maximum of the interval (bt0, et0) */
                normal_(vl, npts);
                break;
      case    2:  /* normalized by the maximum of all interval (b, e)     */
                get_hd_(sac_fil, "depmin", &depmin, &nerr);
                get_hd_(sac_fil, "depmax", &depmax, &nerr);
                get_hd_(sac_fil, "depmen", &depmen, &nerr);
                if(depmin == DEFAULT_FLOAT || depmax == DEFAULT_FLOAT ||
                   depmen == DEFAULT_FLOAT)
                {
                   rd_sac_(sac_fil, ALL, &bt0, &et0, buf, &nv);
                   rmean_(buf, nv, &depmen);
                   maxmin_(buf, nv, &depmin, &depmax, &vlmen);
                }else{
                   depmin -= depmen;
                   depmax -= depmen;
                }
                yy =  fabs(depmin);
                rw = (yy > depmax) ? yy : depmax;
                for(nc = 0; nc < npts; nc++)
                {
                   if(rw != 0.)
                      vl[nc] = (vl[nc] - depmen) / rw;
                }
                break;  
      default:
                break;
   }
} 
get_hd0_(sac_fil, hd_name, hd_buf, nerr)
char     *sac_fil;              /* sac file name            */
char     *hd_name;              /* head variable name       */
void     *hd_buf;               /* head variable buffer     */
int      *nerr;
{
   FILE       *fpin;
   struct    SACHD    shd;
   int      nc, hdn,  nlen;
   int      find;

   find = OFF;
   *nerr = 0;
   if((fpin = fopen(sac_fil, "r")) == NULL)
   {
      fprintf(stderr,  "%s  FILE OPEN ERROR\n", sac_fil);
      exit();
   }
   fread(&shd,sizeof(shd),1,fpin);
   fclose(fpin);
   nlen = strlen(hd_name);
   for(nc = 0; nc < nlen; nc++ )
      hd_name[nc] = tolower(hd_name[nc]);
   for(nc = 0; nc < HDMAX; nc++)
   {
      if(!charcmp(hd_name, sacindx[nc].hdnm))
      {
         find  = ON;
         hdn = sacindx[nc].hdn - 1;
         switch(sacindx[nc].type)   {
            case     FLOAT :
                            memcpy(hd_buf, &shd.r_hd[hdn], 4);
                            break;
            case     INTE4 :
                            memcpy(hd_buf, &shd.n_hd[hdn], 4);
                            break;
            case     CHAR8 :
                            memcpy(hd_buf, shd.a_hd[hdn], 8);
                            break;
            case     CHAR16 :
                            memcpy(hd_buf, shd.a_hd[hdn], 16);
                            break;
            default         :
                            break;
         }
         break;
      }
   }
   if(find == OFF)
   {
      fprintf(stderr, "%s IS NOT A CORRECT HEAD VARIABLE NAME  !!\n",
              hd_name);
      *nerr = -1;
   }
}
/*
 *  =
*/
get_hdF_(sac_fil, r_hd, n_hd, a_hd, nerr)
char    *sac_fil;
float   *r_hd;
int     *n_hd;
char    a_hd[HD_CHAR_LEN][8];
int      *nerr;
{
   FILE       *fpin;
   *nerr = 0;
   if((fpin = fopen(sac_fil, "r")) == NULL)
   {
      fprintf(stderr,  "%s  FILE OPEN ERROR\n", sac_fil);
      *nerr = -1;
      return;
   }
   fread(r_hd, 4, HD_FLOAT_LEN,fpin);
   fread(n_hd, 4, HD_INTE4_LEN,fpin);
   fread(a_hd, 8*HD_CHAR_LEN,1, fpin);
   fclose(fpin);
}
/*
 * get_shd_ =
*/
int get_shd_(sac_fil, shd)
char		*sac_fil;
struct    SACHD    *shd;
{
   FILE       *fpin;
   if((fpin = fopen(sac_fil, "r")) == NULL)
   {
      fprintf(stderr,  "get_shd_: %s  FILE OPEN ERROR\n", sac_fil);
      return(-1);
   }
	if(fread(shd,sizeof(struct SACHD),1,fpin) <1) {
		fprintf(stderr,  "get_shd_: read error\n");
		return(-2);
	}
   fclose(fpin);
	return(0);
}

